#ifdef HAVE_XORG_CONFIG_H
#include <xorg-config.h>
#endif

#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <limits.h>

/* This is based on the code in os/utils.c */

int xdrvswitch(char *cmd)
{
    int pid, p;
#ifdef SIGCHLD
    void (*csig)(int);
#endif
    int status;
    char cmd_path[PATH_MAX];

    sprintf(cmd_path, DRV_SWITCH_PATH "/%s", cmd);
    if (access(cmd_path, X_OK) != 0)
	return 1;

#ifdef SIGCHLD
    csig = signal(SIGCHLD, SIG_DFL);
    if (csig == SIG_ERR) {
      perror("signal");
      return -1;
    }
#endif

    switch (pid = fork()) {
    case -1:	/* error */
	p = -1;
    case 0:	/* child */
	if (setgid(getegid()) == -1)
	    _exit(127);
	if (setuid(geteuid()) == -1)
	    _exit(127);
	execle(cmd_path, cmd_path, NULL, NULL);
	_exit(127);
    default:	/* parent */
	do {
	    p = waitpid(pid, &status, 0);
	} while (p == -1 && errno == EINTR);

    }

#ifdef SIGCHLD
    if (signal(SIGCHLD, csig) == SIG_ERR) {
      perror("signal");
      return -1;
    }
#endif

    return p == -1 ? -1 : status;
}
